@@ -11,7 +11,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkEntry.c,v 1.5.4.1 1999/11/19 23:35:44 hobbs Exp $
+ * RCS: @(#) $Id: tkEntry.c,v 1.14.2.1 2001/04/04 07:57:16 hobbs Exp $
  */
 
 #include "tkInt.h"
@@ -151,6 +151,14 @@ typedef struct {
     int avgWidth;		/* Width of average character. */
     int flags;			/* Miscellaneous flags;  see below for
 				 * definitions. */
+    Tk_TSOffset tsoffset;
+
+    char *validateCmd;          /* Command prefix to use when invoking
+				 * validate command.  NULL means don't
+				 * invoke commands.  Malloc'ed. */
+    int validate;               /* Non-zero means try to validate */
+    char *invalidCmd;		/* Command called when a validation returns 0
+				 * (successfully fails), defaults to {}. */
 } Entry;
 
 /*
@@ -169,6 +177,12 @@ typedef struct {
  * UPDATE_SCROLLBAR:		Non-zero means scrollbar should be updated
  *				during next redisplay operation.
  * GOT_SELECTION:		Non-zero means we've claimed the selection.
+ * ENTRY_DELETED:		This entry has been effectively destroyed.
+ * VALIDATING:                  Non-zero means we are in a validateCmd
+ * VALIDATE_VAR:                Non-zero means we are attempting to validate
+ *                              the entry's textvariable with validateCmd
+ * VALIDATE_ABORT:              Non-zero if validatecommand signals an abort
+ *                              for current procedure and make no changes
  */
 
 #define REDRAW_PENDING		1
@@ -178,6 +192,9 @@ typedef struct {
 #define UPDATE_SCROLLBAR	0x10
 #define GOT_SELECTION		0x20
 #define ENTRY_DELETED           0x40
+#define VALIDATING              0x80
+#define VALIDATE_VAR            0x100
+#define VALIDATE_ABORT          0x200
 
 /*
  * The following macro defines how many extra pixels to leave on each
@@ -201,6 +218,24 @@ static char *stateStrings[] = {
     "disabled", "normal", (char *) NULL
 };
 
+/*
+ * Definitions for -validate option values:
+ */
+
+static char *validateStrings[] = {
+    "all", "key", "focus", "focusin", "focusout", "none", (char *) NULL
+};
+enum validateType {
+    VALIDATE_ALL, VALIDATE_KEY, VALIDATE_FOCUS,
+    VALIDATE_FOCUSIN, VALIDATE_FOCUSOUT, VALIDATE_NONE,
+    /*
+     * These extra enums are for use with EntryValidateChange
+     */
+    VALIDATE_FORCED, VALIDATE_DELETE, VALIDATE_INSERT
+};
+#define DEF_ENTRY_VALIDATE	"none"
+#define DEF_ENTRY_INVALIDCMD	""
+
 /*
  * Information used for argv parsing.
  */
@@ -256,6 +291,11 @@ static Tk_OptionSpec optionSpecs[] = {
     {TK_OPTION_PIXELS, "-insertwidth", "insertWidth", "InsertWidth",
 	DEF_ENTRY_INSERT_WIDTH, -1, Tk_Offset(Entry, insertWidth), 
         0, 0, 0},
+    {TK_OPTION_STRING, "-invalidcommand", "invalidCommand", "InvalidCommand",
+	DEF_ENTRY_INVALIDCMD, -1, Tk_Offset(Entry, invalidCmd),
+	TK_OPTION_NULL_OK, 0, 0},
+    {TK_OPTION_SYNONYM, "-invcmd", (char *) NULL, (char *) NULL,
+	(char *) NULL, 0, -1, 0, (ClientData) "-invalidcommand", 0},
     {TK_OPTION_JUSTIFY, "-justify", "justify", "Justify",
 	DEF_ENTRY_JUSTIFY, -1, Tk_Offset(Entry, justify), 0, 0, 0},
     {TK_OPTION_RELIEF, "-relief", "relief", "Relief",
@@ -279,15 +319,23 @@ static Tk_OptionSpec optionSpecs[] = {
         0, (ClientData) stateStrings, 0},
     {TK_OPTION_STRING, "-takefocus", "takeFocus", "TakeFocus",
 	DEF_ENTRY_TAKE_FOCUS, -1, Tk_Offset(Entry, takeFocus), 
-        TK_CONFIG_NULL_OK, 0, 0},
+        TK_OPTION_NULL_OK, 0, 0},
     {TK_OPTION_STRING, "-textvariable", "textVariable", "Variable",
 	DEF_ENTRY_TEXT_VARIABLE, -1, Tk_Offset(Entry, textVarName),
-	TK_CONFIG_NULL_OK, 0, 0},
+	TK_OPTION_NULL_OK, 0, 0},
+    {TK_OPTION_STRING_TABLE, "-validate", "validate", "Validate",
+       DEF_ENTRY_VALIDATE, -1, Tk_Offset(Entry, validate),
+       0, (ClientData) validateStrings, 0},
+    {TK_OPTION_STRING, "-validatecommand", "validateCommand", "ValidateCommand",
+       (char *) NULL, -1, Tk_Offset(Entry, validateCmd),
+       TK_OPTION_NULL_OK, 0, 0},
+    {TK_OPTION_SYNONYM, "-vcmd", (char *) NULL, (char *) NULL,
+	(char *) NULL, 0, -1, 0, (ClientData) "-validatecommand", 0},
     {TK_OPTION_INT, "-width", "width", "Width",
 	DEF_ENTRY_WIDTH, -1, Tk_Offset(Entry, prefWidth), 0, 0, 0},
     {TK_OPTION_STRING, "-xscrollcommand", "xScrollCommand", "ScrollCommand",
 	DEF_ENTRY_SCROLL_COMMAND, -1, Tk_Offset(Entry, scrollCmd),
-	TK_CONFIG_NULL_OK, 0, 0},
+	TK_OPTION_NULL_OK, 0, 0},
     {TK_OPTION_END, (char *) NULL, (char *) NULL, (char *) NULL,
 	(char *) NULL, 0, -1, 0, 0, 0}
 };
@@ -307,13 +355,13 @@ static Tk_OptionSpec optionSpecs[] = {
 
 static char *commandNames[] = {
     "bbox", "cget", "configure", "delete", "get", "icursor", "index", 
-    "insert", "scan", "selection", "xview", (char *) NULL
+    "insert", "scan", "selection", "validate", "xview", (char *) NULL
 };
 
 enum command {
     COMMAND_BBOX, COMMAND_CGET, COMMAND_CONFIGURE, COMMAND_DELETE, 
     COMMAND_GET, COMMAND_ICURSOR, COMMAND_INDEX, COMMAND_INSERT, 
-    COMMAND_SCAN, COMMAND_SELECTION, COMMAND_XVIEW
+    COMMAND_SCAN, COMMAND_SELECTION, COMMAND_VALIDATE, COMMAND_XVIEW
 };
 
 static char *selCommandNames[] = {
@@ -321,7 +369,7 @@ static char *selCommandNames[] = {
 };
 
 enum selcommand {
-    SELECTION_ADJUST, SELECTION_CLEAR, SELECTION_FROM, 
+    SELECTION_ADJUST, SELECTION_CLEAR, SELECTION_FROM,
     SELECTION_PRESENT, SELECTION_RANGE, SELECTION_TO
 };
 
@@ -358,6 +406,13 @@ static char *		EntryTextVarProc _ANSI_ARGS_((ClientData clientData,
 			    Tcl_Interp *interp, char *name1, char *name2,
 			    int flags));
 static void		EntryUpdateScrollbar _ANSI_ARGS_((Entry *entryPtr));
+static int		EntryValidate _ANSI_ARGS_((Entry *entryPtr,
+			    char *cmd));
+static int		EntryValidateChange _ANSI_ARGS_((Entry *entryPtr,
+			    char *change, char *new, int index, int type));
+static void		ExpandPercents _ANSI_ARGS_((Entry *entryPtr,
+			    char *before, char *change, char *new,
+			    int index, int type, Tcl_DString *dsPtr));
 static void		EntryValueChanged _ANSI_ARGS_((Entry *entryPtr));
 static void		EntryVisibleRange _ANSI_ARGS_((Entry *entryPtr,
 			    double *firstPtr, double *lastPtr));
@@ -448,64 +503,67 @@ Tk_EntryObjCmd(clientData, interp, objc, objv)
      * initialized already (e.g. resource pointers).
      */
 
-    entryPtr = (Entry *) ckalloc(sizeof(Entry));
-    entryPtr->tkwin = tkwin;
-    entryPtr->display = Tk_Display(tkwin);
-    entryPtr->interp = interp;
-    entryPtr->widgetCmd = Tcl_CreateObjCommand(interp,
+    entryPtr			= (Entry *) ckalloc(sizeof(Entry));
+    entryPtr->tkwin		= tkwin;
+    entryPtr->display		= Tk_Display(tkwin);
+    entryPtr->interp		= interp;
+    entryPtr->widgetCmd		= Tcl_CreateObjCommand(interp,
 	    Tk_PathName(entryPtr->tkwin), EntryWidgetObjCmd,
 	    (ClientData) entryPtr, EntryCmdDeletedProc);
-    entryPtr->optionTable = optionTable;
-    entryPtr->string = (char *) ckalloc(1);
-    entryPtr->string[0] = '\0';
-    entryPtr->insertPos = 0;
-    entryPtr->selectFirst = -1;
-    entryPtr->selectLast = -1;
-    entryPtr->selectAnchor = 0;
-    entryPtr->scanMarkX = 0;
-    entryPtr->scanMarkIndex = 0;
+    entryPtr->optionTable	= optionTable;
+    entryPtr->string		= (char *) ckalloc(1);
+    entryPtr->string[0]		= '\0';
+    entryPtr->insertPos		= 0;
+    entryPtr->selectFirst	= -1;
+    entryPtr->selectLast	= -1;
+    entryPtr->selectAnchor	= 0;
+    entryPtr->scanMarkX		= 0;
+    entryPtr->scanMarkIndex	= 0;
 
-    entryPtr->normalBorder = NULL;
-    entryPtr->borderWidth = 0;
-    entryPtr->cursor = None;
-    entryPtr->exportSelection = 1;
-    entryPtr->tkfont = NULL;
-    entryPtr->fgColorPtr = NULL;
-    entryPtr->highlightBgColorPtr = NULL;
-    entryPtr->highlightColorPtr = NULL;
-    entryPtr->highlightWidth = 0;
-    entryPtr->insertBorder = NULL;
-    entryPtr->insertBorderWidth = 0;
-    entryPtr->insertOffTime = 0;
-    entryPtr->insertOnTime = 0;
-    entryPtr->insertWidth = 0;
-    entryPtr->justify = TK_JUSTIFY_LEFT;
-    entryPtr->relief = TK_RELIEF_FLAT;
-    entryPtr->selBorder = NULL;
-    entryPtr->selBorderWidth = 0;
-    entryPtr->selFgColorPtr = NULL;
-    entryPtr->showChar = NULL;
-    entryPtr->state = STATE_NORMAL;
-    entryPtr->textVarName = NULL;
-    entryPtr->takeFocus = NULL;
-    entryPtr->prefWidth = 0;
-    entryPtr->scrollCmd = NULL;
-    entryPtr->numBytes = 0;
-    entryPtr->numChars = 0;
-    entryPtr->displayString = entryPtr->string;
-    entryPtr->numDisplayBytes = 0;
-    entryPtr->inset = XPAD;
-    entryPtr->textLayout = NULL;
-    entryPtr->layoutX = 0;
-    entryPtr->layoutY = 0;
-    entryPtr->leftX = 0;
-    entryPtr->leftIndex = 0;
-    entryPtr->insertBlinkHandler = (Tcl_TimerToken) NULL;
-    entryPtr->textGC = None;
-    entryPtr->selTextGC = None;
-    entryPtr->highlightGC = None;
-    entryPtr->avgWidth = 1;
-    entryPtr->flags = 0;
+    entryPtr->normalBorder	= NULL;
+    entryPtr->borderWidth	= 0;
+    entryPtr->cursor		= None;
+    entryPtr->exportSelection	= 1;
+    entryPtr->tkfont		= NULL;
+    entryPtr->fgColorPtr	= NULL;
+    entryPtr->highlightBgColorPtr	= NULL;
+    entryPtr->highlightColorPtr	= NULL;
+    entryPtr->highlightWidth	= 0;
+    entryPtr->insertBorder	= NULL;
+    entryPtr->insertBorderWidth	= 0;
+    entryPtr->insertOffTime	= 0;
+    entryPtr->insertOnTime	= 0;
+    entryPtr->insertWidth	= 0;
+    entryPtr->justify		= TK_JUSTIFY_LEFT;
+    entryPtr->relief		= TK_RELIEF_FLAT;
+    entryPtr->selBorder		= NULL;
+    entryPtr->selBorderWidth	= 0;
+    entryPtr->selFgColorPtr	= NULL;
+    entryPtr->showChar		= NULL;
+    entryPtr->state		= STATE_NORMAL;
+    entryPtr->textVarName	= NULL;
+    entryPtr->takeFocus		= NULL;
+    entryPtr->prefWidth		= 0;
+    entryPtr->scrollCmd		= NULL;
+    entryPtr->numBytes		= 0;
+    entryPtr->numChars		= 0;
+    entryPtr->displayString	= entryPtr->string;
+    entryPtr->numDisplayBytes	= 0;
+    entryPtr->inset		= XPAD;
+    entryPtr->textLayout	= NULL;
+    entryPtr->layoutX		= 0;
+    entryPtr->layoutY		= 0;
+    entryPtr->leftX		= 0;
+    entryPtr->leftIndex		= 0;
+    entryPtr->insertBlinkHandler	= (Tcl_TimerToken) NULL;
+    entryPtr->textGC		= None;
+    entryPtr->selTextGC		= None;
+    entryPtr->highlightGC	= None;
+    entryPtr->avgWidth		= 1;
+    entryPtr->flags		= 0;
+    entryPtr->validateCmd	= NULL;
+    entryPtr->validate		= VALIDATE_NONE;
+    entryPtr->invalidCmd	= NULL;
 
     Tk_SetClass(entryPtr->tkwin, "Entry");
     TkSetClassProcs(entryPtr->tkwin, &entryClass, (ClientData) entryPtr);
@@ -515,21 +573,15 @@ Tk_EntryObjCmd(clientData, interp, objc, objv)
     Tk_CreateSelHandler(entryPtr->tkwin, XA_PRIMARY, XA_STRING,
 	    EntryFetchSelection, (ClientData) entryPtr, XA_STRING);
 
-    if (Tk_InitOptions(interp, (char *) entryPtr, optionTable, tkwin)
-	    != TCL_OK) {
+    if ((Tk_InitOptions(interp, (char *) entryPtr, optionTable, tkwin)
+	    != TCL_OK) ||
+	    (ConfigureEntry(interp, entryPtr, objc-2, objv+2, 0) != TCL_OK)) {
 	Tk_DestroyWindow(entryPtr->tkwin);
 	return TCL_ERROR;
     }
-    if (ConfigureEntry(interp, entryPtr, objc-2, objv+2, 0) != TCL_OK) {
-	goto error;
-    }
     
     Tcl_SetResult(interp, Tk_PathName(entryPtr->tkwin), TCL_STATIC);
     return TCL_OK;
-
-    error:
-    Tk_DestroyWindow(entryPtr->tkwin);
-    return TCL_ERROR;
 }
 
 /*
@@ -581,11 +633,10 @@ EntryWidgetObjCmd(clientData, interp, objc, objv)
     switch (cmdIndex) {
         case COMMAND_BBOX: {
 	    int index, x, y, width, height;
-	    char *string;
 	    char buf[TCL_INTEGER_SPACE * 4];
 
 	    if (objc != 3) {
-	        Tcl_WrongNumArgs(interp, 1, objv, "bbox index");
+	        Tcl_WrongNumArgs(interp, 2, objv, "index");
 		goto error;
 	    }
 	    if (GetEntryIndex(interp, entryPtr, Tcl_GetString(objv[2]), 
@@ -595,7 +646,6 @@ EntryWidgetObjCmd(clientData, interp, objc, objv)
 	    if ((index == entryPtr->numChars) && (index > 0)) {
 	        index--;
 	    }
-	    string = entryPtr->displayString;
 	    Tk_CharBbox(entryPtr->textLayout, index, &x, &y, 
                     &width, &height);
 	    sprintf(buf, "%d %d %d %d", x + entryPtr->layoutX,
@@ -606,7 +656,7 @@ EntryWidgetObjCmd(clientData, interp, objc, objv)
 	
         case COMMAND_CGET: {
 	    if (objc != 3) {
-	        Tcl_WrongNumArgs(interp, 1, objv, "cget option");
+	        Tcl_WrongNumArgs(interp, 2, objv, "option");
 		goto error;
 	    }
 	    
@@ -641,8 +691,7 @@ EntryWidgetObjCmd(clientData, interp, objc, objv)
 	    int first, last;
 
 	    if ((objc < 3) || (objc > 4)) {
-	        Tcl_WrongNumArgs(interp, 1, objv, 
-                        "delete firstIndex ?lastIndex?");
+	        Tcl_WrongNumArgs(interp, 2, objv, "firstIndex ?lastIndex?");
 		goto error;
 	    }
 	    if (GetEntryIndex(interp, entryPtr, Tcl_GetString(objv[2]), 
@@ -665,7 +714,7 @@ EntryWidgetObjCmd(clientData, interp, objc, objv)
 
         case COMMAND_GET: {
 	    if (objc != 2) {
-	        Tcl_WrongNumArgs(interp, 1, objv, "get");
+	        Tcl_WrongNumArgs(interp, 2, objv, (char *) NULL);
 		goto error;
 	    }
 	    Tcl_SetResult(interp, entryPtr->string, TCL_STATIC);
@@ -674,7 +723,7 @@ EntryWidgetObjCmd(clientData, interp, objc, objv)
 
         case COMMAND_ICURSOR: {
 	    if (objc != 3) {
-	        Tcl_WrongNumArgs(interp, 1, objv, "icursor pos");
+	        Tcl_WrongNumArgs(interp, 2, objv, "pos");
 		goto error;
 	    }
 	    if (GetEntryIndex(interp, entryPtr, Tcl_GetString(objv[2]),
@@ -687,18 +736,16 @@ EntryWidgetObjCmd(clientData, interp, objc, objv)
 	
         case COMMAND_INDEX: {
 	    int index;
-	    char buf[TCL_INTEGER_SPACE];
 
 	    if (objc != 3) {
-	        Tcl_WrongNumArgs(interp, 1, objv, "index string");
+	        Tcl_WrongNumArgs(interp, 2, objv, "string");
 		goto error;
 	    }
 	    if (GetEntryIndex(interp, entryPtr, Tcl_GetString(objv[2]), 
                     &index) != TCL_OK) {
 	        goto error;
 	    }
-	    sprintf(buf, "%d", index);
-	    Tcl_SetResult(interp, buf, TCL_VOLATILE);
+	    Tcl_SetObjResult(interp, Tcl_NewIntObj(index));
 	    break;
 	}
 
@@ -706,7 +753,7 @@ EntryWidgetObjCmd(clientData, interp, objc, objv)
 	    int index;
 
 	    if (objc != 4) {
-	        Tcl_WrongNumArgs(interp, 1, objv, "insert index text");
+	        Tcl_WrongNumArgs(interp, 2, objv, "index text");
 		goto error;
 	    }
 	    if (GetEntryIndex(interp, entryPtr, Tcl_GetString(objv[2]), 
@@ -724,7 +771,7 @@ EntryWidgetObjCmd(clientData, interp, objc, objv)
 	    char *minorCmd;
 
 	    if (objc != 4) {
-	        Tcl_WrongNumArgs(interp, 1, objv, "scan mark|dragto x");
+	        Tcl_WrongNumArgs(interp, 2, objv, "mark|dragto x");
 		goto error;
 	    }
 	    if (Tcl_GetIntFromObj(interp, objv[3], &x) != TCL_OK) {
@@ -752,7 +799,7 @@ EntryWidgetObjCmd(clientData, interp, objc, objv)
 	    int index, index2;
 
 	    if (objc < 3) {
-	        Tcl_WrongNumArgs(interp, 1, objv, "select option ?index?");
+	        Tcl_WrongNumArgs(interp, 2, objv, "option ?index?");
 		goto error;
 	    }
 
@@ -770,8 +817,7 @@ EntryWidgetObjCmd(clientData, interp, objc, objv)
 	    switch(selIndex) {
 	        case SELECTION_ADJUST: {
 		    if (objc != 4) {
-		        Tcl_WrongNumArgs(interp, 1, objv, 
-                                "selection adjust index");
+		        Tcl_WrongNumArgs(interp, 3, objv, "index");
 			goto error;
 		    }
 		    if (GetEntryIndex(interp, entryPtr, 
@@ -802,7 +848,7 @@ EntryWidgetObjCmd(clientData, interp, objc, objv)
 
 	        case SELECTION_CLEAR: {
 		    if (objc != 3) {
-		        Tcl_WrongNumArgs(interp, 1, objv, "selection clear");
+		        Tcl_WrongNumArgs(interp, 3, objv, (char *) NULL);
 			goto error;
 		    }
 		    if (entryPtr->selectFirst >= 0) {
@@ -815,8 +861,7 @@ EntryWidgetObjCmd(clientData, interp, objc, objv)
 
 	        case SELECTION_FROM: {
 		    if (objc != 4) {
-		        Tcl_WrongNumArgs(interp, 1, objv, 
-			        "selection from index");
+		        Tcl_WrongNumArgs(interp, 3, objv, "index");
 			goto error;
 		    }
 		    if (GetEntryIndex(interp, entryPtr, 
@@ -829,7 +874,7 @@ EntryWidgetObjCmd(clientData, interp, objc, objv)
 
 	        case SELECTION_PRESENT: {
 		    if (objc != 3) {
-		        Tcl_WrongNumArgs(interp, 1, objv, "selection present");
+		        Tcl_WrongNumArgs(interp, 3, objv, (char *) NULL);
 			goto error;
 		    }
 		    if (entryPtr->selectFirst < 0) {
@@ -842,8 +887,7 @@ EntryWidgetObjCmd(clientData, interp, objc, objv)
 
 	        case SELECTION_RANGE: {
 		    if (objc != 5) {
-		        Tcl_WrongNumArgs(interp, 1, objv, 
-                                "selection range start end");
+		        Tcl_WrongNumArgs(interp, 3, objv, "start end");
 			goto error;
 		    }
 		    if (GetEntryIndex(interp, entryPtr, 
@@ -873,8 +917,7 @@ EntryWidgetObjCmd(clientData, interp, objc, objv)
 		
 	        case SELECTION_TO: {
 		    if (objc != 4) {
-		        Tcl_WrongNumArgs(interp, 1, objv, 
-                                "selection to index");
+		        Tcl_WrongNumArgs(interp, 3, objv, "index");
 			goto error;
 		    }
 		    if (GetEntryIndex(interp, entryPtr, 
@@ -888,6 +931,24 @@ EntryWidgetObjCmd(clientData, interp, objc, objv)
 	    break;
 	}
 
+        case COMMAND_VALIDATE: {
+	    int code;
+
+	    if (objc != 2) {
+		Tcl_WrongNumArgs(interp, 2, objv, (char *) NULL);
+		goto error;
+	    }
+	    selIndex = entryPtr->validate;
+	    entryPtr->validate = VALIDATE_ALL;
+	    code = EntryValidateChange(entryPtr, (char *) NULL,
+				       entryPtr->string, -1, VALIDATE_FORCED);
+	    if (entryPtr->validate != VALIDATE_NONE) {
+		entryPtr->validate = selIndex;
+	    }
+	    Tcl_SetObjResult(interp, Tcl_NewBooleanObj((code == TCL_OK)));
+	    break;
+	}
+
         case COMMAND_XVIEW: {
 	    int index;
 
@@ -982,12 +1043,6 @@ DestroyEntry(memPtr)
     char *memPtr;		/* Info about entry widget. */
 {
     Entry *entryPtr = (Entry *) memPtr;
-    entryPtr->flags |= ENTRY_DELETED;
-
-    Tcl_DeleteCommandFromToken(entryPtr->interp, entryPtr->widgetCmd);
-    if (entryPtr->flags & REDRAW_PENDING) {
-        Tcl_CancelIdleCall(DisplayEntry, (ClientData) entryPtr);
-    }
 
     /*
      * Free up all the stuff that requires special handling, then
@@ -1015,6 +1070,14 @@ DestroyEntry(memPtr)
     Tk_FreeConfigOptions((char *) entryPtr, entryPtr->optionTable,
 	    entryPtr->tkwin);
     entryPtr->tkwin = NULL;
+
+    /*
+     * Tcl_EventuallyFree should be used here or better yet in the
+     * DestroyNotify branch of EntryEventProc.  However, that can lead
+     * complications in Tk_FreeConfigOptions where the display for the
+     * entry has been deleted by Tk_DestroyWindow, which is needed
+     * when freeing the cursor option. 
+     */
     ckfree((char *) entryPtr);
 }
 
@@ -1102,11 +1165,15 @@ ConfigureEntry(interp, entryPtr, objc, objv, flags)
 
 	/*
 	 * Restart the cursor timing sequence in case the on-time or 
-	 * off-time just changed.
+	 * off-time just changed.  Set validate temporarily to none,
+	 * so the configure doesn't cause it to be triggered.
 	 */
 
 	if (entryPtr->flags & GOT_FOCUS) {
-	  EntryFocusProc(entryPtr, 1);
+	    int validate = entryPtr->validate;
+	    entryPtr->validate = VALIDATE_NONE;
+	    EntryFocusProc(entryPtr, 1);
+	    entryPtr->validate = validate;
 	}
 
 	/*
@@ -1192,7 +1259,7 @@ EntryWorldChanged(instanceData)
     ClientData instanceData;	/* Information about widget. */
 {
     XGCValues gcValues;
-    GC gc;
+    GC gc = None;
     unsigned long mask;
     Entry *entryPtr;
 
@@ -1203,6 +1270,10 @@ EntryWorldChanged(instanceData)
 	entryPtr->avgWidth = 1;
     }
 
+    if (entryPtr->normalBorder != NULL) {
+	Tk_SetBackgroundFromBorder(entryPtr->tkwin, entryPtr->normalBorder);
+    }
+
     gcValues.foreground = entryPtr->fgColorPtr->pixel;
     gcValues.font = Tk_FontId(entryPtr->tkfont);
     gcValues.graphics_exposures = False;
@@ -1255,14 +1326,12 @@ DisplayEntry(clientData)
     Entry *entryPtr = (Entry *) clientData;
     Tk_Window tkwin = entryPtr->tkwin;
     int baseY, selStartX, selEndX, cursorX;
-    int xBound;
+    int showSelection, xBound;
     Tk_FontMetrics fm;
     Pixmap pixmap;
-    int showSelection;
-    char *string;
 
     entryPtr->flags &= ~REDRAW_PENDING;
-    if ((entryPtr->tkwin == NULL) || !Tk_IsMapped(tkwin)) {
+    if ((entryPtr->flags & ENTRY_DELETED) || !Tk_IsMapped(tkwin)) {
 	return;
     }
 
@@ -1277,6 +1346,15 @@ DisplayEntry(clientData)
 	EntryUpdateScrollbar(entryPtr);
     }
 
+    /*
+     * We do this check twice because updating the scrollbar can have
+     * the side-effect of destroying or unmapping the entry widget.
+     */
+
+    if ((entryPtr->flags & ENTRY_DELETED) || !Tk_IsMapped(tkwin)) {
+	return;
+    }
+
     /*
      * In order to avoid screen flashes, this procedure redraws the
      * textual area of the entry into off-screen memory, then copies
@@ -1313,9 +1391,8 @@ DisplayEntry(clientData)
      */
 
     Tk_Fill3DRectangle(tkwin, pixmap, entryPtr->normalBorder,
-	    0, 0, Tk_Width(tkwin), Tk_Height(tkwin), 0, TK_RELIEF_FLAT);
+		0, 0, Tk_Width(tkwin), Tk_Height(tkwin), 0, TK_RELIEF_FLAT);
 
-    string = entryPtr->displayString;
     if (showSelection
 	    && (entryPtr->selectLast > entryPtr->leftIndex)) {
 	if (entryPtr->selectFirst <= entryPtr->leftIndex) {
@@ -1599,6 +1676,14 @@ InsertChars(entryPtr, index, value)
     strcpy(new + byteIndex, value);
     strcpy(new + byteIndex + byteCount, string + byteIndex);
 
+    if ((entryPtr->validate == VALIDATE_KEY ||
+	 entryPtr->validate == VALIDATE_ALL) &&
+	EntryValidateChange(entryPtr, value, new, index,
+			    VALIDATE_INSERT) != TCL_OK) {
+	ckfree(new);
+	return;
+    }
+
     ckfree(string);
     entryPtr->string = new;
 
@@ -1673,7 +1758,7 @@ DeleteChars(entryPtr, index, count)
     int count;			/* How many characters to delete. */
 {
     int byteIndex, byteCount, newByteCount;
-    char *new, *string;
+    char *new, *string, *todelete;
 
     if ((index + count) > entryPtr->numChars) {
 	count = entryPtr->numChars - index;
@@ -1691,6 +1776,20 @@ DeleteChars(entryPtr, index, count)
     memcpy(new, string, (size_t) byteIndex);
     strcpy(new + byteIndex, string + byteIndex + byteCount);
 
+    todelete = (char *) ckalloc((unsigned) (byteCount + 1));
+    memcpy(todelete, string + byteIndex, (size_t) byteCount);
+    todelete[byteCount] = '\0';
+
+    if ((entryPtr->validate == VALIDATE_KEY ||
+	 entryPtr->validate == VALIDATE_ALL) &&
+	EntryValidateChange(entryPtr, todelete, new, index,
+			    VALIDATE_DELETE) != TCL_OK) {
+	ckfree(new);
+	ckfree(todelete);
+	return;
+    }
+
+    ckfree(todelete);
     ckfree(entryPtr->string);
     entryPtr->string = new;
     entryPtr->numChars -= count;
@@ -1830,15 +1929,52 @@ EntrySetValue(entryPtr, value)
     char *value;		/* New text to display in entry. */
 {
     char *oldSource;
+    int code, valueLen, malloced = 0;
+
+    if (strcmp(value, entryPtr->string) == 0) {
+	return;
+    }
+    valueLen = strlen(value);
+
+    if (entryPtr->flags & VALIDATE_VAR) {
+	entryPtr->flags |= VALIDATE_ABORT;
+    } else {
+	/*
+	 * If we validate, we create a copy of the value, as it may
+	 * point to volatile memory, like the value of the -textvar
+	 * which may get freed during validation
+	 */
+	oldSource = (char *) ckalloc((unsigned) (valueLen + 1));
+	strcpy(oldSource, value);
+	value = oldSource;
+	malloced = 1;
+
+	entryPtr->flags |= VALIDATE_VAR;
+	code = EntryValidateChange(entryPtr, (char *) NULL, value, -1,
+		VALIDATE_FORCED);
+	entryPtr->flags &= ~VALIDATE_VAR;
+	/*
+	 * If VALIDATE_ABORT has been set, then this operation should be
+	 * aborted because the validatecommand did something else instead
+	 */
+	if (entryPtr->flags & VALIDATE_ABORT) {
+	    entryPtr->flags &= ~VALIDATE_ABORT;
+	    ckfree(value);
+	    return;
+	}
+    }
 
     oldSource = entryPtr->string;
-
     ckfree(entryPtr->string);
-    entryPtr->numBytes = strlen(value);
-    entryPtr->numChars = Tcl_NumUtfChars(value, entryPtr->numBytes);
-    entryPtr->string =
-	    (char *) ckalloc((unsigned) (entryPtr->numBytes + 1));
-    strcpy(entryPtr->string, value);
+
+    if (malloced) {
+	entryPtr->string = value;
+    } else {
+	entryPtr->string   = (char *) ckalloc((unsigned) (valueLen + 1));
+	strcpy(entryPtr->string, value);
+    }
+    entryPtr->numBytes = valueLen;
+    entryPtr->numChars = Tcl_NumUtfChars(value, valueLen);
 
     if (entryPtr->displayString == oldSource) {
 	entryPtr->displayString = entryPtr->string;
@@ -1897,7 +2033,15 @@ EntryEventProc(clientData, eventPtr)
 	EventuallyRedraw(entryPtr);
 	entryPtr->flags |= BORDER_NEEDED;
     } else if (eventPtr->type == DestroyNotify) {
-        DestroyEntry((char *) clientData);
+	if (!(entryPtr->flags & ENTRY_DELETED)) {
+	    entryPtr->flags |= (ENTRY_DELETED | VALIDATE_ABORT);
+	    Tcl_DeleteCommandFromToken(entryPtr->interp,
+		    entryPtr->widgetCmd);
+	    if (entryPtr->flags & REDRAW_PENDING) {
+		Tcl_CancelIdleCall(DisplayEntry, clientData);
+	    }
+	    DestroyEntry((char *) entryPtr);
+	}
     } else if (eventPtr->type == ConfigureNotify) {
 	Tcl_Preserve((ClientData) entryPtr);
 	entryPtr->flags |= UPDATE_SCROLLBAR;
@@ -2305,7 +2449,7 @@ static void
 EventuallyRedraw(entryPtr)
     Entry *entryPtr;		/* Information about widget. */
 {
-    if ((entryPtr->tkwin == NULL) || !Tk_IsMapped(entryPtr->tkwin)) {
+    if ((entryPtr->flags & ENTRY_DELETED) || !Tk_IsMapped(entryPtr->tkwin)) {
 	return;
     }
 
@@ -2442,7 +2586,8 @@ EntryBlinkProc(clientData)
 {
     Entry *entryPtr = (Entry *) clientData;
 
-    if (!(entryPtr->flags & GOT_FOCUS) || (entryPtr->insertOffTime == 0)) {
+    if ((entryPtr->state == STATE_DISABLED) ||
+	    !(entryPtr->flags & GOT_FOCUS) || (entryPtr->insertOffTime == 0)) {
 	return;
     }
     if (entryPtr->flags & CURSOR_ON) {
@@ -2489,9 +2634,21 @@ EntryFocusProc(entryPtr, gotFocus)
 		    entryPtr->insertOnTime, EntryBlinkProc,
 		    (ClientData) entryPtr);
 	}
+	if (entryPtr->validate == VALIDATE_ALL ||
+	    entryPtr->validate == VALIDATE_FOCUS ||
+	    entryPtr->validate == VALIDATE_FOCUSIN) {
+	    EntryValidateChange(entryPtr, (char *) NULL,
+				entryPtr->string, -1, VALIDATE_FOCUSIN);
+	}
     } else {
 	entryPtr->flags &= ~(GOT_FOCUS | CURSOR_ON);
 	entryPtr->insertBlinkHandler = (Tcl_TimerToken) NULL;
+	if (entryPtr->validate == VALIDATE_ALL ||
+	    entryPtr->validate == VALIDATE_FOCUS ||
+	    entryPtr->validate == VALIDATE_FOCUSOUT) {
+	    EntryValidateChange(entryPtr, (char *) NULL,
+				entryPtr->string, -1, VALIDATE_FOCUSOUT);
+	}
     }
     EventuallyRedraw(entryPtr);
 }
@@ -2553,8 +2710,335 @@ EntryTextVarProc(clientData, interp, name1, name2, flags)
     if (value == NULL) {
 	value = "";
     }
-    if (strcmp(value, entryPtr->string) != 0) {
-	EntrySetValue(entryPtr, value);
-    }
+    EntrySetValue(entryPtr, value);
     return (char *) NULL;
 }
+
+/*
+ *--------------------------------------------------------------
+ *
+ * EntryValidate --
+ *
+ *	This procedure is invoked when any character is added or
+ *	removed from the entry widget, or a focus has trigerred validation.
+ *
+ * Results:
+ *	TCL_OK if the validatecommand passes the new string.
+ *      TCL_BREAK if the vcmd executed OK, but rejects the string.
+ *      TCL_ERROR if an error occurred while executing the vcmd
+ *      or a valid Tcl_Bool is not returned.
+ *
+ * Side effects:
+ *      An error condition may arise
+ *
+ *--------------------------------------------------------------
+ */
+
+static int
+EntryValidate(entryPtr, cmd)
+     register Entry *entryPtr;	/* Entry that needs validation. */
+     register char *cmd;	/* Validation command (NULL-terminated
+				 * string). */
+{
+    register Tcl_Interp *interp = entryPtr->interp;
+    int code, bool;
+
+    code = Tcl_EvalEx(interp, cmd, -1, TCL_EVAL_GLOBAL | TCL_EVAL_DIRECT);
+
+    /*
+     * We accept TCL_OK and TCL_RETURN as valid return codes from the
+     * command callback.
+     */
+    if (code != TCL_OK && code != TCL_RETURN) {
+	Tcl_AddErrorInfo(interp,
+			 "\n\t(in validation command executed by entry)");
+	Tcl_BackgroundError(interp);
+	return TCL_ERROR;
+    }
+
+    /*
+     * The command callback should return an acceptable Tcl boolean.
+     */
+    if (Tcl_GetBooleanFromObj(interp, Tcl_GetObjResult(interp),
+			      &bool) != TCL_OK) {
+	Tcl_AddErrorInfo(interp,
+		 "\nvalid boolean not returned by validation command");
+	Tcl_BackgroundError(interp);
+	Tcl_SetResult(interp, NULL, 0);
+	return TCL_ERROR;
+    }
+
+    Tcl_SetResult(interp, NULL, 0);
+    return (bool ? TCL_OK : TCL_BREAK);
+}
+
+/*
+ *--------------------------------------------------------------
+ *
+ * EntryValidateChange --
+ *
+ *	This procedure is invoked when any character is added or
+ *	removed from the entry widget, or a focus has trigerred validation.
+ *
+ * Results:
+ *	TCL_OK if the validatecommand accepts the new string,
+ *      TCL_ERROR if any problems occured with validatecommand.
+ *
+ * Side effects:
+ *      The insertion/deletion may be aborted, and the
+ *      validatecommand might turn itself off (if an error
+ *      or loop condition arises).
+ *
+ *--------------------------------------------------------------
+ */
+
+static int
+EntryValidateChange(entryPtr, change, new, index, type)
+     register Entry *entryPtr;	/* Entry that needs validation. */
+     char *change;		/* Characters to be added/deleted
+				 * (NULL-terminated string). */
+     char *new;                 /* Potential new value of entry string */
+     int index;                 /* index of insert/delete, -1 otherwise */
+     int type;                  /* forced, delete, insert,
+				 * focusin or focusout */
+{
+    int code, varValidate = (entryPtr->flags & VALIDATE_VAR);
+    char *p;
+    Tcl_DString script;
+    
+    if (entryPtr->validateCmd == NULL ||
+	entryPtr->validate == VALIDATE_NONE) {
+	return (varValidate ? TCL_ERROR : TCL_OK);
+    }
+
+    /*
+     * If we're already validating, then we're hitting a loop condition
+     * Return and set validate to 0 to disallow further validations
+     * and prevent current validation from finishing
+     */
+    if (entryPtr->flags & VALIDATING) {
+	entryPtr->validate = VALIDATE_NONE;
+	return (varValidate ? TCL_ERROR : TCL_OK);
+    }
+
+    entryPtr->flags |= VALIDATING;
+
+    /*
+     * Now form command string and run through the -validatecommand
+     */
+
+    Tcl_DStringInit(&script);
+    ExpandPercents(entryPtr, entryPtr->validateCmd,
+	    change, new, index, type, &script);
+    Tcl_DStringAppend(&script, "", 1);
+
+    p = Tcl_DStringValue(&script);
+    code = EntryValidate(entryPtr, p);
+    Tcl_DStringFree(&script);
+
+    /*
+     * If e->validate has become VALIDATE_NONE during the validation, or
+     * we now have VALIDATE_VAR set (from EntrySetValue) and didn't before,
+     * it means that a loop condition almost occured.  Do not allow
+     * this validation result to finish.
+     */
+
+    if (entryPtr->validate == VALIDATE_NONE
+	    || (!varValidate && (entryPtr->flags & VALIDATE_VAR))) {
+	code = TCL_ERROR;
+    }
+
+    /*
+     * It's possible that the user deleted the entry during validation.
+     * In that case, abort future validation and return an error.
+     */
+
+    if (entryPtr->flags & ENTRY_DELETED) {
+	return TCL_ERROR;
+    }
+
+    /*
+     * If validate will return ERROR, then disallow further validations
+     * Otherwise, if it didn't accept the new string (returned TCL_BREAK)
+     * then eval the invalidCmd (if it's set)
+     */
+
+    if (code == TCL_ERROR) {
+	entryPtr->validate = VALIDATE_NONE;
+    } else if (code == TCL_BREAK) {
+	/*
+	 * If we were doing forced validation (like via a variable
+	 * trace) and the command returned 0, the we turn off validation
+	 * because we assume that textvariables have precedence in
+	 * managing the value.  We also don't call the invcmd, as it
+	 * may want to do entry manipulation which the setting of the
+	 * var will later wipe anyway.
+	 */
+	if (varValidate) {
+	    entryPtr->validate = VALIDATE_NONE;
+	} else if (entryPtr->invalidCmd != NULL) {
+	    Tcl_DStringInit(&script);
+	    ExpandPercents(entryPtr, entryPtr->invalidCmd,
+			   change, new, index, type, &script);
+	    Tcl_DStringAppend(&script, "", 1);
+	    p = Tcl_DStringValue(&script);
+	    if (Tcl_EvalEx(entryPtr->interp, p, -1,
+		    TCL_EVAL_GLOBAL | TCL_EVAL_DIRECT) != TCL_OK) {
+		Tcl_AddErrorInfo(entryPtr->interp,
+				 "\n\t(in invalidcommand executed by entry)");
+		Tcl_BackgroundError(entryPtr->interp);
+		code = TCL_ERROR;
+		entryPtr->validate = VALIDATE_NONE;
+	    }
+	    Tcl_DStringFree(&script);
+
+	    /*
+	     * It's possible that the user deleted the entry during validation.
+	     * In that case, abort future validation and return an error.
+	     */
+
+	    if (entryPtr->flags & ENTRY_DELETED) {
+		return TCL_ERROR;
+	    }
+	}
+    }
+
+    entryPtr->flags &= ~VALIDATING;
+
+    return code;
+}
+
+/*
+ *--------------------------------------------------------------
+ *
+ * ExpandPercents --
+ *
+ *	Given a command and an event, produce a new command
+ *	by replacing % constructs in the original command
+ *	with information from the X event.
+ *
+ * Results:
+ *	The new expanded command is appended to the dynamic string
+ *	given by dsPtr.
+ *
+ * Side effects:
+ *	None.
+ *
+ *--------------------------------------------------------------
+ */
+
+static void
+ExpandPercents(entryPtr, before, change, new, index, type, dsPtr)
+     register Entry *entryPtr;	/* Entry that needs validation. */
+     register char *before;	/* Command containing percent
+				 * expressions to be replaced. */
+     char *change;		/* Characters to added/deleted
+				 * (NULL-terminated string). */
+     char *new;			/* Potential new value of entry string */
+     int index;			/* index of insert/delete */
+     int type;			/* INSERT or DELETE */
+     Tcl_DString *dsPtr;	/* Dynamic string in which to append
+				 * new command. */
+{
+    int spaceNeeded, cvtFlags;	/* Used to substitute string as proper Tcl
+				 * list element. */
+    int number, length;
+    register char *string;
+    Tcl_UniChar ch;
+    char numStorage[2*TCL_INTEGER_SPACE];
+
+    while (1) {
+	if (*before == '\0') {
+	    break;
+	}
+	/*
+	 * Find everything up to the next % character and append it
+	 * to the result string.
+	 */
+
+	string = before;
+	/* No need to convert '%', as it is in ascii range */
+	string = Tcl_UtfFindFirst(before, '%');
+	if (string == (char *) NULL) {
+	    Tcl_DStringAppend(dsPtr, before, -1);
+	    break;
+	} else if (string != before) {
+	    Tcl_DStringAppend(dsPtr, before, string-before);
+	    before = string;
+	}
+
+	/*
+	 * There's a percent sequence here.  Process it.
+	 */
+
+	before++; /* skip over % */
+	if (*before != '\0') {
+	    before += Tcl_UtfToUniChar(before, &ch);
+	} else {
+	    ch = '%';
+	}
+	switch (ch) {
+	case 'd': /* Type of call that caused validation */
+	    switch (type) {
+	    case VALIDATE_INSERT:
+		number = 1;
+		break;
+	    case VALIDATE_DELETE:
+		number = 0;
+		break;
+	    default:
+		number = -1;
+		break;
+	    }
+	    sprintf(numStorage, "%d", number);
+	    string = numStorage;
+	    break;
+	case 'i': /* index of insert/delete */
+	    sprintf(numStorage, "%d", index);
+	    string = numStorage;
+	    break;
+	case 'P': /* 'Peeked' new value of the string */
+	    string = new;
+	    break;
+	case 's': /* Current string value of entry */
+	    string = entryPtr->string;
+	    break;
+	case 'S': /* string to be inserted/deleted, if any */
+	    string = change;
+	    break;
+	case 'v': /* type of validation currently set */
+	    string = validateStrings[entryPtr->validate];
+	    break;
+	case 'V': /* type of validation in effect */
+	    switch (type) {
+	    case VALIDATE_INSERT:
+	    case VALIDATE_DELETE:
+		string = validateStrings[VALIDATE_KEY];
+		break;
+	    case VALIDATE_FORCED:
+		string = "forced";
+		break;
+	    default:
+		string = validateStrings[type];
+		break;
+	    }
+	    break;
+	case 'W': /* widget name */
+	    string = Tk_PathName(entryPtr->tkwin);
+	    break;
+	default:
+	    length = Tcl_UniCharToUtf(ch, numStorage);
+	    numStorage[length] = '\0';
+	    string = numStorage;
+	    break;
+	}
+
+	spaceNeeded = Tcl_ScanCountedElement(string, -1, &cvtFlags);
+	length = Tcl_DStringLength(dsPtr);
+	Tcl_DStringSetLength(dsPtr, length + spaceNeeded);
+	spaceNeeded = Tcl_ConvertCountedElement(string, -1,
+		Tcl_DStringValue(dsPtr) + length,
+		cvtFlags | TCL_DONT_USE_BRACES);
+	Tcl_DStringSetLength(dsPtr, length + spaceNeeded);
+    }
+}
