@@ -13,7 +13,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkTest.c,v 1.5 1999/04/16 01:51:23 stanton Exp $
+ * RCS: @(#) $Id: tkTest.c,v 1.12 2000/04/04 08:09:15 hobbs Exp $
  */
 
 #include "tkInt.h"
@@ -61,10 +61,17 @@ typedef struct TImageInstance {
  * The type record for test images:
  */
 
+#ifdef USE_OLD_IMAGE
 static int		ImageCreate _ANSI_ARGS_((Tcl_Interp *interp,
 			    char *name, int argc, char **argv,
 			    Tk_ImageType *typePtr, Tk_ImageMaster master,
 			    ClientData *clientDataPtr));
+#else
+static int		ImageCreate _ANSI_ARGS_((Tcl_Interp *interp,
+			    char *name, int argc, Tcl_Obj *CONST objv[],
+			    Tk_ImageType *typePtr, Tk_ImageMaster master,
+			    ClientData *clientDataPtr));
+#endif
 static ClientData	ImageGet _ANSI_ARGS_((Tk_Window tkwin,
 			    ClientData clientData));
 static void		ImageDisplay _ANSI_ARGS_((ClientData clientData,
@@ -78,11 +85,12 @@ static void		ImageDelete _ANSI_ARGS_((ClientData clientData));
 
 static Tk_ImageType imageType = {
     "test",			/* name */
-    ImageCreate,		/* createProc */
+    (Tk_ImageCreateProc *) ImageCreate, /* createProc */
     ImageGet,			/* getProc */
     ImageDisplay,		/* displayProc */
     ImageFree,			/* freeProc */
     ImageDelete,		/* deleteProc */
+    (Tk_ImagePostscriptProc *) NULL,/* postscriptPtr */
     (Tk_ImageType *) NULL	/* nextPtr */
 };
 
@@ -202,8 +210,7 @@ static void		TrivialEventProc _ANSI_ARGS_((ClientData clientData,
  * External (platform specific) initialization routine:
  */
 
-extern int		TkplatformtestInit _ANSI_ARGS_((
-			    Tcl_Interp *interp));
+extern int		TkplatformtestInit _ANSI_ARGS_((Tcl_Interp *interp));
 extern int              TclThread_Init _ANSI_ARGS_((Tcl_Interp *interp));
 
 #if !(defined(__WIN32__) || defined(MAC_TCL))
@@ -1556,6 +1563,7 @@ TestfontObjCmd(clientData, interp, objc, objv)
  */
 
 	/* ARGSUSED */
+#ifdef USE_OLD_IMAGE
 static int
 ImageCreate(interp, name, argc, argv, typePtr, master, clientDataPtr)
     Tcl_Interp *interp;		/* Interpreter for application containing
@@ -1574,20 +1582,55 @@ ImageCreate(interp, name, argc, argv, typePtr, master, clientDataPtr)
     char *varName;
     int i;
 
+    Tk_InitImageArgs(interp, argc, &argv);
     varName = "log";
     for (i = 0; i < argc; i += 2) {
 	if (strcmp(argv[i], "-variable") != 0) {
-	    Tcl_AppendResult(interp, "bad option name \"", argv[i],
-		    "\"", (char *) NULL);
+	    Tcl_AppendResult(interp, "bad option name \"",
+		    argv[i], "\"", (char *) NULL);
 	    return TCL_ERROR;
 	}
 	if ((i+1) == argc) {
-	    Tcl_AppendResult(interp, "no value given for \"", argv[i],
-		    "\" option", (char *) NULL);
+	    Tcl_AppendResult(interp, "no value given for \"",
+		    argv[i], "\" option", (char *) NULL);
 	    return TCL_ERROR;
 	}
 	varName = argv[i+1];
     }
+#else
+static int
+ImageCreate(interp, name, objc, objv, typePtr, master, clientDataPtr)
+    Tcl_Interp *interp;		/* Interpreter for application containing
+				 * image. */
+    char *name;			/* Name to use for image. */
+    int objc;			/* Number of arguments. */
+    Tcl_Obj *CONST objv[];	/* Argument strings for options (doesn't
+				 * include image name or type). */
+    Tk_ImageType *typePtr;	/* Pointer to our type record (not used). */
+    Tk_ImageMaster master;	/* Token for image, to be used by us in
+				 * later callbacks. */
+    ClientData *clientDataPtr;	/* Store manager's token for image here;
+				 * it will be returned in later callbacks. */
+{
+    TImageMaster *timPtr;
+    char *varName;
+    int i;
+
+    varName = "log";
+    for (i = 0; i < objc; i += 2) {
+	if (strcmp(Tcl_GetString(objv[i]), "-variable") != 0) {
+	    Tcl_AppendResult(interp, "bad option name \"",
+		    Tcl_GetString(objv[i]), "\"", (char *) NULL);
+	    return TCL_ERROR;
+	}
+	if ((i+1) == objc) {
+	    Tcl_AppendResult(interp, "no value given for \"",
+		    Tcl_GetString(objv[i]), "\" option", (char *) NULL);
+	    return TCL_ERROR;
+	}
+	varName = Tcl_GetString(objv[i+1]);
+    }
+#endif
     timPtr = (TImageMaster *) ckalloc(sizeof(TImageMaster));
     timPtr->master = master;
     timPtr->interp = interp;
