@@ -11,7 +11,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * RCS: @(#) $Id: tkMacResource.r,v 1.4 1999/04/16 01:51:32 stanton Exp $
+ * RCS: @(#) $Id: tkMacResource.r,v 1.6.2.1 2001/04/04 21:39:49 hobbs Exp $
  */
 
 /*
@@ -46,20 +46,6 @@
 #   define MINOR_VERSION TK_MINOR_VERSION * 16
 #endif
 
-resource 'vers' (1) {
-	TK_MAJOR_VERSION, MINOR_VERSION,
-	RELEASE_LEVEL, 0x00, verUS,
-	TK_PATCH_LEVEL,
-	TK_PATCH_LEVEL ", by Ray Johnson  1993-1996" "\n" "Sun Microsystems Labratories"
-};
-
-resource 'vers' (2) {
-	TK_MAJOR_VERSION, MINOR_VERSION,
-	RELEASE_LEVEL, 0x00, verUS,
-	TK_PATCH_LEVEL,
-	"Wish " TK_PATCH_LEVEL "  1993-1996"
-};
-
 
 /* 
  * The mechanisim below loads Tcl source into the resource fork of the
@@ -108,11 +94,29 @@ read 'TEXT' (26, "comdlg", purgeable, preload) "::library:comdlg.tcl";
  */
 		
 resource 'STR#' (128, "Tcl Environment Variables") {
-	{	"SCHEDULE_NAME=Agent Controller Schedule",
+	{	
+		/*		
+		"SCHEDULE_NAME=Agent Controller Schedule",
 		"SCHEDULE_PATH=Lozoya:System Folder:Tcl Lib:Tcl-Scheduler"
+		*/
 	};
 };
 
+data 'alis' (1000, "Library Folder") {
+	$"0000 0000 00BA 0002 0001 012F 0000 0000"            /* ........../.... */
+	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
+	$"0000 0000 0000 985C FB00 4244 0000 0000"            /* ......\.BD.... */
+	$"0002 1328 5375 7070 6F72 7420 4C69 6272"            /* ...(Support Libr */
+	$"6172 6965 7329 0000 0000 0000 0000 0000"            /* aries).......... */
+	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
+	$"0000 0000 0000 0000 0000 0000 0000 0000"            /* ................ */
+	$"0000 0076 8504 B617 A796 003D 0027 025B"            /* ...v...=.'.[ */
+	$"01E4 0001 0001 0000 0000 0000 0000 0000"            /* ............... */
+	$"0000 0000 0000 0000 0001 2F00 0002 0015"            /* ........../..... */
+	$"2F3A 2853 7570 706F 7274 204C 6962 7261"            /* /:(Support Libra */
+	$"7269 6573 2900 FFFF 0000"                           /* ries)... */
+};
+
 /*
  * The following two resources define the default "About Box" for Mac Tk.
  * This dialog appears if the "About Tk..." menu item is selected from
@@ -134,8 +138,8 @@ resource 'DITL' (128, "About Box", purgeable) {
 	    "Wish - Windowing Shell" "\n" "based on Tcl " 
 	    TCL_PATCH_LEVEL " & Tk " TK_PATCH_LEVEL "\n\n" 
             "Ray Johnson & Jim Ingham" "\n"
-	    "Sun Microsystems Labs" "\n" "ray.johnson@eng.sun.com"
-            "\n" "jim.ingham@eng.sun.com"},	    
+	    "Scriptics Inc." "\n"
+            "jim.ingham@cygnus.com"},	    
         { 19,  24, 119,  92}, Picture  {enabled, 128}
     }
 };
